import Gio from 'gi://Gio';
import GLib from 'gi://GLib';
import Soup from 'gi://Soup';

Gio._promisify(Soup.Session.prototype, 'send_and_read_async');
Gio._promisify(Gio.File.prototype, 'replace_contents_bytes_async', 'replace_contents_finish');

/**
 *
 * @param {Extension} extension
 * @param {Soup.Session} session
 */
export async function getIP(extension, session) {
    const message = Soup.Message.new_from_encoded_form(
        'GET',
        'https://ipinfo.io/ip',
        Soup.form_encode_hash(extension.soupParams)
    );

    let info;
    try {
        const bytes = await session.send_and_read_async(message, GLib.PRIORITY_DEFAULT, null);

        const decoder = new TextDecoder('utf-8');
        info = decoder.decode(bytes.get_data());
        if (message.statusCode === Soup.Status.OK) {
            return [info, null];
        } else {
            console.log(`IP-Finder getIp() failed with status code - ${message.statusCode}`);
            return [null, info];
        }
    } catch (e) {
        console.log(`IP-Finder getIp() error - ${e}`);
        return [null, e];
    }
}

/**
 *
 * @param {Extension} extension
 * @param {Soup.Session} session
 * @param {string} ipAddr
 */
export async function getIPDetails(extension, session, ipAddr) {
    const message = Soup.Message.new_from_encoded_form(
        'GET',
        `https://ipinfo.io/${ipAddr}/json`,
        Soup.form_encode_hash(extension.soupParams)
    );

    let info;
    try {
        const bytes = await session.send_and_read_async(message, GLib.PRIORITY_DEFAULT, null);

        const decoder = new TextDecoder('utf-8');
        info = JSON.parse(decoder.decode(bytes.get_data()));
        if (message.statusCode === Soup.Status.OK) {
            return [info, null];
        } else {
            console.log(`IP-Finder getIpDetails() failed with status code - ${message.statusCode}`);
            return [null, `${message.statusCode} - ${info['error'].title}. ${info['error'].message}.`];
        }
    } catch (e) {
        console.log(`IP-Finder getIpDetails() error - ${e}`);
        return [null, `IP-Finder getIpDetails() error - ${e}`];
    }
}

/**
 *
 * @param {Extension} extension
 * @param {Soup.Session} session
 */
export async function getIPDetails2(extension, session) {
    const getParams = {
        id: `ip-finder/'${extension.metadata.version}`,
        fields: '63479',
    };
    const message = Soup.Message.new_from_encoded_form(
        'GET',
        'http://ip-api.com/json/',
        Soup.form_encode_hash(getParams)
    );

    let info;
    try {
        const bytes = await session.send_and_read_async(message, GLib.PRIORITY_DEFAULT, null);

        const decoder = new TextDecoder('utf-8');
        info = JSON.parse(decoder.decode(bytes.get_data()));
        if (message.statusCode === Soup.Status.OK) {
            return [info, null];
        } else {
            console.log(`IP-Finder getIpDetails() failed with status code - ${message.statusCode}`);
            return [null, `${message.statusCode} - ${info['message']}.`];
        }
    } catch (e) {
        console.log(`IP-Finder getIpDetails() error - ${e}`);
        return [null, `IP-Finder getIpDetails() error - ${e}`];
    }
}

/**
 *
 * @param {Array} coordinates
 * @param {int} zoom
 */
export function getMapTileInfo(coordinates, zoom) {
    const [lat, lon] = coordinates.split(',').map(Number);
    const xTile = Math.floor((lon + 180.0) / 360.0 * (1 << zoom));
    const yTile = Math.floor((1.0 - Math.log(Math.tan(lat * Math.PI / 180.0) + 1.0 / Math.cos(lat * Math.PI / 180.0)) / Math.PI) / 2.0 * (1 << zoom));

    return {zoom, xTile, yTile};
}

/**
 *
 * @param {Extension} extension
 * @param {Soup.Session} session
 * @param {string} tileInfo
 */
export async function getMapTile(extension, session, tileInfo) {
    const file = Gio.file_new_for_path(`${extension.path}/icons/latest_map.png`);

    const message = Soup.Message.new_from_encoded_form(
        'GET',
        `https://a.tile.openstreetmap.org/${tileInfo}.png`,
        Soup.form_encode_hash(extension.soupParams)
    );

    let info;
    try {
        const bytes = await session.send_and_read_async(message, GLib.PRIORITY_DEFAULT, null);

        if (message.statusCode === Soup.Status.OK) {
            info = bytes.get_data();
            const [success, etag_] = await file.replace_contents_bytes_async(info, null, false, Gio.FileCreateFlags.REPLACE_DESTINATION, null);
            return success ? [file, null] : [null, 'Error replacing map tile file.'];
        } else {
            console.log(`IP-Finder getMapTile() failed with status code - ${message.statusCode}`);
            return [null, message.statusCode];
        }
    } catch (e) {
        console.log(`IP-Finder getMapTile() error - ${e}`);
        return [null, message.statusCode];
    }
}
